# Generated by Django 4.2.9 on 2024-02-20 10:08

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wbcrm", "0007_alter_account_status"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="activity",
            options={"verbose_name": "Activity", "verbose_name_plural": "Activities"},
        ),
    ]
