# Generated by Django 4.1.10 on 2023-08-16 08:16

import django_fsm
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wbcrm", "0003_remove_activity_external_id_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="activity",
            name="status",
            field=django_fsm.FSMField(
                choices=[
                    ("CANCELLED", "Cancelled"),
                    ("PLANNED", "Planned"),
                    ("FINISHED", "Finished"),
                    ("REVIEWED", "Reviewed"),
                ],
                default="PLANNED",
                max_length=50,
                verbose_name="Status",
            ),
        ),
    ]
