# Generated by Django 5.0.12 on 2025-10-15 13:24

import django.contrib.postgres.search
import django.contrib.postgres.indexes
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('wbcrm', '0017_event'),
    ]

    operations = [
        migrations.AddField(
            model_name='activity',
            name='search_vector',
            field=django.contrib.postgres.search.SearchVectorField(null=True),
        ),
        migrations.AddIndex(
            model_name='activity',
            index=django.contrib.postgres.indexes.GinIndex(fields=['search_vector'], name='activity_sv_gin_idx'),
        ),
    ]
