# Generated by Django 5.0.7 on 2024-08-02 13:37

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbcrm", "0009_alter_account_is_public"),
    ]

    operations = [
        migrations.AlterField(
            model_name="account",
            name="reference_id",
            field=models.PositiveIntegerField(blank=True, unique=True),
        ),
    ]
