# Generated by Django 5.0.9 on 2024-10-18 13:46

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbcrm", "0011_activity_summary"),
    ]

    operations = [
        migrations.AlterField(
            model_name="activity",
            name="summary",
            field=models.TextField(blank=True, default="", verbose_name="LLM Summary"),
        ),
    ]
